#
#  ICONS support
#                               by Hidetoshi NAGAI (nagai@ai.kyutech.ac.jp)
#

# call setup script for general 'tkextlib' libraries
require 'tkextlib/setup.rb'

# call setup script
require 'tkextlib/ICONS/setup.rb'

# load library
require 'tkextlib/ICONS/icons'
